/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package moka.util;

import moka.lang.Object;

/**
 * The string tokenizer class allows an application to break a
 * string into tokens.
 * The delimiters (the characters that separate tokens) may
 * be specified either at creation time or on a per-token basis.
 *
 * @author  Frdric Brown
 * @version 1.0, 2002-11-08
 * @since   MDK1.0a
 */
public class StringTokenizer extends moka.lang.Object {
	/** An altered copy of the original string.*/
	private String str = null;
	/** A copy of the original string.*/
	private String origin = null;
	/** The index of the current char in the string. Should be only read.*/
	public int currentPosition;


    /**
     * Constructs a string tokenizer for the specified string. The
     * tokenizer uses the default delimiters, which is
     * the space character,
     * the tab character, the newline character, the carriage-return character,
     * and the form-feed character. If the string object risk to be used later, use a copy of the string object
	 * instead of the original, the string being finalized before the method returns.
     *
     * @param   str   a string to be parsed.
     */
    public StringTokenizer (String str) {
    }

    /**
     * Constructs a string tokenizer for the specified string. The
     * characters in the <code>delim</code> argument is the delimiters
     * for separating tokens. If the string objects risk to be used later, use a copy of the string objects
	 * instead of the originals, the strings being finalized before the method returns.
     *
     * @param   str     a string to be parsed.
     * @param   delim   the delimiter.
     */
    public StringTokenizer (String str, String delim) {
    }

    /**
     * Frees the memory of system ressources used by this StringTokenizer
     * object.
     *
     */
 	public native void finalize() ;

    /**
     * Tests if there are more tokens available from this tokenizer's string.
     * If this method returns <tt>true</tt>, then a subsequent call to
     * <tt>nextToken</tt> with no argument will successfully return a token.
     *
     * @return  <code>true</code> if and only if there is at least one token
     *          in the string after the current position; <code>false</code>
     *          otherwise.
     */
    public native boolean hasMoreTokens () ;

    /**
     * Returns the next token from this string tokenizer.
     *
     * @return     the next token from this string tokenizer.
     */
    public String nextToken() {

    	return new String();
    }

    /**
     * Returns the next token in this string tokenizer's string. First,
     * the set of characters considered to be delimiters by this
     * <tt>StringTokenizer</tt> object is changed to be the characters in
     * the string <te recognized tt>delim</tt>. Then the next token in the string
     * after the current position is returned. The current position is
     * advanced beyond thoken.  The new delimiter set
     * remains the default after this call. If the string object risk to be used later, use a copy of the string object
	 * instead of the original, the string being finalized before the method returns.
     *
     * @param      delim   the new delimiters.
     * @return     the next token, after switching to the new delimiter set.
     */
    public String nextToken(String delim) {

    	return this.nextToken();
    }

    /**
     * Calculates the number of times that this tokenizer's
     * <code>nextToken</code> method can be called. The current
     * position is not advanced.
     *
     * @return  the number of tokens remaining in the string using the current
     *          delimiter set.
     */
    public int countTokens() {
    	return 0;
    }

    /**
     * Sets the current position to the specified index.
     *
     * currentPosition the new current position.
     */
    public void setCurrentPosition(int currentPosition) {
    }
}