/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package moka.x;

import moka.x.GEM;
import moka.x.Captioned;
import moka.x.Container;

/**
 * Class Frame is a floating window.
 *
 * @author  Frdric Brown
 * @version 1.1, 2002-11-06
 * @since   MDK1.0a
 */
 public class Frame extends Container implements Captioned {
	/** Indicate that the default close operation exits the application.*/
	public static final short EXIT_ON_CLOSE = 0;
	/** Indicate that the default close operation disposes of this frame.*/
	public static final short DISPOSE_ON_CLOSE = 1;
	/** Indicate that the default close operation hides the frame.*/
	public static final short HIDE_ON_CLOSE = 2;
	/** Indicate that the default close operation does nothing.*/
	public static final short DO_NOTHING_ON_CLOSE = 3;

	/** Is this frame movable ?*/
	public boolean movable;

	/** Is this frame modal ?  Should be only read.*/
	public boolean modal;

	/** The default close operation for this frame.*/
	public short defaultCloseOperation;

	/** Is the close button visible ?  Should be only read.*/
	public boolean closeButtonVisible;

	/** Is the minimize button visible ?  Should be only read.*/
	public boolean minimizeButtonVisible;

	/*
	 * Constructs a new frame.
	 */
	public Frame () {
	}

	/*
	 * Constructs a new frame specifying its caption.
	 *
	 * @param caption the frame's caption.
	 */
	public Frame (String caption) {
	}

    /**
     * Free the memory and system ressources used by this Frame.
     */
	public void finalize () {
	}

	/**
	 * This method is invoked by the GEM when the user interacts
	 * with the frame.
	 */
	public void use () {
	}

	/**
	 * Indicates if the cursor is over the close button of this frame.
	 *
	 * @param x the reference's horizontal coordinate
	 * @param y	the reference's vertical coordinate
	 * @return true if the cursor is over the close button, false otherwize.
	 */
	public boolean isCursorOverCloseButton (short x, short y) {
		return false;
	}

	/**
	 * Indicates if the cursor is over the minimize button of this frame.
	 *
	 * @param x the reference's horizontal coordinate
	 * @param y	the reference's vertical coordinate
	 * @return true if the cursor is over the minimize button, false otherwize.
	 */
	public boolean isCursorOverMinimizeButton (short x, short y) {
		return false;
	}

	/**
	 * Indicates if the cursor is over the title bar of this frame.
	 *
	 * @param x the reference's horizontal coordinate
	 * @param y	the reference's vertical coordinate
	 * @return true if the cursor is over the title bar, false otherwize.
	 */
	public boolean isCursorOverTitleBar (short x, short y) {
		return false;
	}

	/**
	 * Called when a CloseEvent occurs.  The method will perform a task
	 * according to the default close operation of this frame.  A CloseEvent
	 * occurs when the user uses the close button of the frame.
	 */
	public void onClose () {
	}

	/**
	 * Called when a MinimizeEvent occurs.  The method will minimize the window
	 * to the task bar of the desktop.  A MinimizeEvent
	 * occurs when the user uses the minimize button of the frame.
	 */
	public void onMinimize () {
	}

	/**
	 * Called when a DragEvent occurs.  The method will notify the GEM
	 * with a DragEvent.  A DragEvent
	 * occurs when the user click on the title bar.
	 */
	public void onDrag () {
	}

	/**
	 * Disposes of all ressources used by this frame and
	 * all its children.
	 */
	public void dispose () {
	}

	/**
	 * Sets the visibility of the frame.
	 *
	 * @param b true if the frame should be visible, false otherwize.
	 */
	public void setVisible (boolean b) {
	}

	/**
	 * Sets frame is modal or not.
	 *
	 * @param b true if the frame should be modal, false otherwize.
	 */
	public void setModal (boolean b) {
	}

	/**
	 * Sets the visibility of the close button.
	 *
	 * @param b true if the close button should be visible, false otherwize.
	 */
	public void setCloseButtonVisible (boolean b) {
	}

	/**
	 * Sets the visibility of the minimize button.
	 *
	 * @param b true if the minimize button should be visible, false otherwize.
	 */
	public void setMinimizeButtonVisible (boolean b) {
	}

	/**
	 * Paints the frame.
	 */
	public native void paint () ;

	/**
	 * Paints the foreground of the frame.
	 */
	public native void paintForeground () ;

	/**
	 * Indicates if the cursor is over this frame.
	 *
	 * @param x the reference's horizontal coordinate
	 * @param y	the reference's vertical coordinate
	 * @return true if the cursor is over the frame, false otherwize.
	 */
	public boolean isCursorOver (short x, short y) {

		return false;
	}
}