/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */
 
/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.System;
import moka.lang.Integer;

import moka.io.Serializable;
import moka.io.File;

/**
 * This class is an example of program that uses the Serializable class.
 *
 * @author  Frdric Brown
 */
public class CSerial extends Serializable {
	/** Some fields.*/
	public int age;
	public String name;
	public CSerial parent;
	
	public static void main () {
		File f = new File("test");//Instanciate a new File to manipulate a file named "test"
		CSerial obj;//A reference to an Object
		
		System.clrscr();//Clear the screen
		
		System.print("Please enter age: ");//Show a message to the user
		
		obj = new CSerial(Integer.parseInt(System.readLine()));//Instanciate a new Serial with an integer parsed from the user's input
		
		System.println();//New line
		
		System.print("Please enter name: ");//Show a message to the user
		
		obj.name = System.readLine();//Assign the name
		
		System.println();//New line

		System.print("Parent age: ");//Show a message to the user
		
		obj.parent = new CSerial(Integer.parseInt(System.readLine()));//Instanciate a new Serial with an integer parsed from the user's input
		
		System.println();//New line
		
		System.print("Parent name: ");//Show a message to the user
		
		obj.parent.name = System.readLine();//Assign the name
		
		System.println();//New line
		
		f.open(File.MODE_WRITE);//Open the file in write mode
		
		f.writeObject(obj);//Writes the object
		
		obj.finalize();//Finalize the original object
		
		f.close();//Close the file
		
		f.open(File.MODE_OPEN_READ);//Open the file in read mode
		
		obj = f.readObject();//Read an object from the file
		
		f.close();//Close the file
		
		System.println(obj);//Print the object read from the file (should be the same that the saved object)
		
		obj.finalize();//Finalize the read object
		
		f.finalize();//Finalize the file object
		
		System.read();//Wait for the user to press a key
	}
	
	/**
	 * Construct a new Serial, initializing age to the
	 * specified value.
	 * @param age The desired value.
	 */
	public CSerial (int age) {
		this.age = age;
		this.parent = null;
		this.name = null;
	}
	/**
	 * Frees the resources used by this Serial
	 */
	public void finalize () {
		if (this.name) {
			this.name.finalize();
		}
		
		if (this.parent) {
			this.parent.finalize();
		}
		
		super.finalize();
	}
	
	/**
	 * Returns a String representation of this Serial.
	 * In fact return the name and age of this Serial and of its parent.
	 *
	 * @return a String representation of this Serial.
	 */
	public String toString () {
		String str = this.name.toString();//Instanciate a String using the name of this serial
		
		str += ", age: "; //Concatenate the string ", age: "
		
		str += this.age; // Concatenate the age
		
		str += "\n Parent : "; //Concatenate the string "\n Parent : "
		
		if (this.parent) {
			/* This Serial has a parent */
			str += this.parent.name.toString();//Instanciate a String using the name of this serial
			str += ", age: "; //Concatenate the string ", age: "	
			str += this.parent.age; // Concatenate the age
		}
		
		return str;
	}
}