/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.lang.String;//Import the class moka.lang.String
import moka.io.Keyboard;//Import the class moka.io.Keyboard

/**
 * This class represents a player of the Tron game.
 *
 * @author  Frdric Brown
 */
public class Player {
	/** Defines no direction.*/
	public static final short NO_DIRECTION = -1;
	/** Defines the up direction.*/
	public static final short UP_DIRECTION = 0;
	/** Defines the down direction.*/
	public static final short DOWN_DIRECTION = 1;
	/** Defines the left direction.*/
	public static final short LEFT_DIRECTION = 2;
	/** Defines the right direction.*/
	public static final short RIGHT_DIRECTION = 3;

	/** The score of the player.*/
	private short points;
	/** The dirrection of the player.*/
	private short direct;
	/** The x position of the player.*/
	private short x;
	/** The y position of the player.*/
	private short y;
	/** The number of the player.*/
	private short no;

	/**
	 * Initializes a newly allocated Player.
	 *
	 * @param no The number of the player
	 */
	public Player (short no) {
		this.points = 0;//A Player begins with 0 points
		this.no = no;//Assign the no parameter to the no of this Player
	}

	/**
	 * Prepares the Player for a new round.
	 *
	 * @param x The initial x position of the player
	 * @param y The initial y position of the player
	 */
	public void respawn (short x, short y) {
		this.direct = Player.NO_DIRECTION;//A Player begins with no direction
		this.x = x;//Assign the x parameter to the x of this Player
		this.y = y;//Assign the y parameter to the y of this Player
	}

	/**
	 * Draws the player at its actual position
	 */
	public void draw () {
		Graphics.drawPix(this.x, this.y, Graphics.NORMAL_ATTR);
	}

	/**
	 * Returns a String representation of the Player.
	 * The string returned is the concatenation of the
	 * String "Player " followed by the number of the player
	 * then by the String " : " and finnaly by the score of
	 * the player has.
	 *
	 * @return A String representation of the Player
	 */
	public String toString() {
		String str = "Player ";//Declare str as a new String with a initial value of "Player "

		str.concat(String.valueOf(this.no));//Append the number of the Player
		str.concat(" : ");//Append " : "
		return str.concat(String.valueOf(this.points));//Append the score of the Player then return the String
	}

	/**
	 * An abstract method that reads the keyboard to set the new direction
	 * of the Player.
	 *
	 * @param kb The Keyboard to read
	 */
	public abstract void readKb (Keyboard kb);

	/**
	 * Returns the direction of the Player.
	 *
	 * @return the direction of the Player
	 */
	public short getDirect () {
		return this.direct;
	}

	/**
	 * Move the Player in accordance of its direction.
	 */
	public void move () {
		switch (this.direct) {
			case 0: {//if the player direction is up
				this.y = this.y - 1;//decrement the y coordinate of the Player
				break;
			}
			case 1: {//if the player direction is down
				this.y = this.y + 1;//increment the y coordinate of the Player
				break;
			}
			case 2: {//if the player direction is left
				this.x = this.x - 1;//decrement the x coordinate of the Player
				break;
			}
			case 3: {//if the player direction is right
				this.x = this.x + 1;//increment the x coordinate of the Player
				break;
			}
		}
	}

	/**
	 * Returns the score of the Player.
	 *
	 * @return The score of the Player
	 */
	public short getPoints() {
		return this.points;
	}

	/**
	 * Checks if the player hits a wall.
	 *
	 * @return true if the player hits a wall, false otherwize
	 */
	public boolean hit () {
		return Graphics.getPixGray(this.x, this.y);
	}

	/**
	 * Called when a Player wins a round. Increment the score of a player by 1.
	 */
	public void win () {
		Graphics.setGray(Gray.BLACK);//Set the grayscale level to BLACK
		String str = "Player ";
		str.concat(String.valueOf(this.no));
		str.concat(" wins the round !");
		this.points++;
		Graphics.draw((short)0, (short)0, str, Graphics.REPLACE_ATTR);//Draw the String "Player (player no) wins the round !"
	}
}