/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka.
 *
 *  Moka is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Moka; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import moka.x.Frame;
import moka.x.Canvas;
import moka.x.RadioButton;
import moka.x.RadioGroup;
import moka.x.Button;
import moka.x.OptionPane;

/**
 * This class represents a drawing program.
 *
 * @author  Frdric Brown
 */
public class Draw extends Frame {
	/** A Canvas.*/
	private Canvas canvas;

	/** A Checkbox.*/
	private RadioButton black;

	/** A Checkbox.*/
	private RadioButton white;

	/** A Checkbox.*/
	private RadioButton invert;

	/** A Button.*/
	private Button btnErase;

	/** The RadioGroup.*/
	private RadioGroup rGroup;

	/**
	 * Constructs a new Draw frame.
	 */
	public Draw () {
		super("Draw your pic");//Call the superclass' constructor, passing the string "Draw your pic" as argument

		this.setBounds(0s, 0s, 150s, 100s);//Set the bounds of this frame to x=0, y=0, width=150 and height=100
		this.setMinimizeButtonVisible(false);//Hide the minimize button of the title bar
		this.defaultCloseOperation = Frame.EXIT_ON_CLOSE;//Set the default close operation for this frame to EXIT ON CLOSE

		this.rGroup = new RadioGroup();//Instantiate a new RadioGroup

		this.canvas = new Canvas();//Instanciate a new Canvas
		this.canvas.setBounds(2s, 15s, 140s, 60s);//Set the bounds of the img canvas to x=2, y=80, width=140 and height=60
		this.add(this.canvas);//Add canvas into this frame

		this.black = new RadioButton("B");//Instantiate a new RadioButton labeled "B"
		this.black.setBounds(2s, 80s, 30s, 17s);//Set the bounds of the black button to x=2, y=80, width=30 and height=17
		this.black.setMnemonic('B');//Set the mnemonic for the black button to 'B'
		this.black.listen = this;//Set the listener of black to this
		this.black.setGroup(this.rGroup);//Sets the RadioGroup of black to this.rGroup
		this.add(this.black);//Add black into this frame

		this.white = new RadioButton("W");//Instantiate a new RadioButton labeled "W"
		this.white.setBounds(34s, 80s, 30s, 17s);//Set the bounds of the white button to x=34, y=80, width=30 and height=17
		this.white.setMnemonic('W');//Set the mnemonic for the white button to 'W'
		this.white.listen = this;//Set the listener of white to this
		this.white.setGroup(this.rGroup);//Sets the RadioGroup of white to this.rGroup
		this.add(this.white);//Add white into this frame

		this.invert = new RadioButton("I");//Instantiate a new RadioButton labeled "I"
		this.invert.setBounds(66s, 80s, 30s, 17s);//Set the bounds of the invert button to x=66, y=80, width=30 and height=17
		this.invert.setMnemonic('I');//Set the mnemonic for the invert button to 'I'
		this.invert.listen = this;//Set the listener of invert to this
		this.invert.setGroup(this.rGroup);//Sets the RadioGroup of invert to this.rGroup
		this.add(this.invert);//Add invert into this frame

		this.btnErase = new Button("Erase");//Instantiate a new Button labeled "Erase"
		this.btnErase.setBounds(98s, 80s, 50s, 17s);//Set the bounds of the btnErase button to x=98, y=80, width=40 and height=17
		this.btnErase.setMnemonic('E');//Set the mnemonic for the btnErase button to 'E'
		this.btnErase.listen = this;//Set the listener of btnErase to this
		this.add(this.btnErase);//Add btnErase into this frame

		this.black.use();//Select the black button
	}

	/**
	 * Called when a CloseEvent occurs.  Will display a confirmation message.
	 */
	public void onClose() {
		String name = OptionPane.showInputDialog((Container)null, "Save", "File name ?");//Ask the file name to the user

		if (name) {
			this.canvas.saveToFile(name);
		}

		super.onClose();//Call the superclass onClose method : dispose of the frame.
	}

	/**
	 * Called when an Event occurs.
	 */
	public void eventTriggered (Object sender) {
		if (sender == this.black) {//If the sender is the black button
			this.canvas.color = Color.BLACK_COLOR;//Set the pen's color to black
		}
		else if (sender == this.white) {//If the sender is the white button
			this.canvas.color = Color.WHITE_COLOR;//Set the pen's color to white
		}
		else if (sender == this.invert) {//If the sender is the invert button
			this.canvas.color = Color.INVERT_COLOR;//Set the pen's color to invert
		}
		else if (sender == this.btnErase) {//If the sender is the btnErase button
			this.canvas.erase();//Erase the canvas
		}
	}
	
    /**
     * Frees the ressources used by this object.
     */
	public void finalize () {
		this.rGroup.finalize();
		super.finalize();
	}
	
	/**
	 * The main static method, the entry point of the program.
	 */
	public static void main () {
		Draw f = new Draw();//Instantiate a new Draw Frame

		f.setVisible(true);//Show the Draw frame
	}
}