/*
 * QUSOFT MICROSYSTMES
 * Moka
 * Copyright 2002 Frdric Brown
 */

/*
 *  This file is part of Moka API.
 *
 *  Moka API is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Moka API is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Moka API; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package moka.x;

import moka.x.CaptionedComponent;
import moka.util.StringTokenizer;
import moka.lang.System;

/**
 * TextArea class is a text entry component.
 *
 * @author  Frdric Brown
 * @version 1.1, 2003-03-10
 * @since   MDK1.0a
 */
public class TextArea extends CaptionedComponent {
	/** The number of character this TextField should accept. Should be > 0.*/
	public short maxLength;

	/** Should the words be warped ? Should be only read.*/
	public boolean warp;

	/*
	 * Constructs a new TextArea.
	 */
	public TextArea () {
		this("");
	}

	/*
	 * Constructs a new TextArea specifying its caption.
	 *
	 * @param caption the TextArea's caption.
	 */
	public TextArea (String caption) {
		super(caption);
		this.maxLength = 2048;
		this.warp = true;
	}

	/**
	 * Sets if the words should be warped.
	 *
	 * @param b true to specify that the words should be warped, false to specify that the words should not be warped
	 */
	public void setWarp(boolean b) {
		this.warp = b;
	}

	/**
	 * Paints the TextArea.
	 */
	public native void paint() {
		if (this->font != Font_FONT_SYSTEM) {
			FontSetSys(this->font);
		}

		if (this->warp) {
			this->printTextWarp_char_p_short_int_short_int(this, this->caption->value, 0, 0);
		}
		else {
			this->printTextNoWarp_char_p_short_int_short_int_short_int(this, this->caption->value, 0, 0, -1);
		}

		if (this->font != Font_FONT_SYSTEM) {
			FontSetSys(GEM_gem->systemFont);
		}

		DrawClipRect (&(WIN_RECT){0, 0,  this->width - 1, this->height - 1}, GEM_vScrRect, A_NORMAL);
	}

	/**
	 * Prints the text in this TextArea while warping words.
	 *
	 * @param str The text to print
	 * @param x The x position
	 * @param y The y position
	 */
	private native void printTextWarp (char* str, short x, short y) {
		short i;
		short n;
		short z;
		short pos = 0;
		short width = GEM_getFontWidth_short_int(this->font);
		short height = GEM_getFontHeight_short_int(this->font) + 1;
		short nbL = this->height / height;
		short nbC = (this->width - 3) / width;
		short len = strlen(str);
		short l;
		TStringTokenizer* tok = TStringTokenizer_String(TString_char_p(str));
		TString* s;

		for (i = 0; i < nbL && pos < len; i++) {
			for (n = 0; n < nbC && pos < len; n++) {
				if (str[pos] == ' ') {
					pos++;
					if (n < 1) {
						n--;
					}
				}
				else if (str[pos] == '\n') {
					pos++;
					i++;
					n = -1;
				}
				else {
					s = tok->nextToken_(tok);

					l = strlen(s->value);

					pos += l;

					if (l + n >= nbC && n > 0) {
						i++;
						n = 0;
					}

					for (z = 0; z < l && i < nbL; z++) {


						DrawChar(x + n * width + 2, y + i * height + 2, s->value[z], A_NORMAL);

						if (z + 1 < l) {
							if (n + 1 >= nbC)  {
								i++;
								n = 0;
							}
							else {
								n++;
							}
						}
					}

					s->finalize_(s);
				}
			}
		}

		tok->finalize_(tok);
	}

	/**
	 * Prints the text in this TextArea without warping words.
	 *
	 * @param str The text to print
	 * @param x The x position
	 * @param y The y position
	 * @param cur The current position
	 */
	private native void printTextNoWarp (char* str, short x, short y, short cur) {
		short i;
		short n = 0;
		short pos = 0;
		short width = GEM_getFontWidth_short_int(this->font);
		short height = GEM_getFontHeight_short_int(this->font) + 1;
		short nbL = this->height / height;
		short nbC = (this->width - 5) / width;
		short len = strlen(str);

		for (i = 0; i < nbL && pos < len; i++) {
			for (n = 0; n < nbC && pos < len; n++) {
				if (str[pos] == '\n') {
					if (pos == cur) {
						DrawChar(x + n * width + 2, y + i * height + 2, ' ', A_REVERSE);
					}
					pos++;
					n = nbC;
				}
				else if (pos != cur) {
					DrawChar(x + n * width + 2, y + i * height + 2, str[pos++], A_NORMAL);
				}
				else {
					DrawChar(x + n * width + 2, y + i * height + 2, str[pos++], A_REVERSE);
				}
			}
		}

		if ( pos == 0  ) {
			DrawChar ( x + n * width + 2 , y + i * height + 2 , ' ' , A_REVERSE ) ;
			
		}
		else if (pos == len && pos == cur) {
			
			if (n >= nbC) {
				n = 0;
			}
			else {
				i--;
			}
			DrawChar ( x + n * width + 2 , y + i * height + 2 , ' ' , A_REVERSE ) ;
		}
	}

	/**
	 * This method is invoked by the GEM when the user interacts
	 * with the TextArea.
	 */
	public void use () {
	    /*char buffer[this.maxLength + 1];
	    char btemp[this.maxLength + 1];*/
	    char[] buffer = new char[this.maxLength + 1];
	    char[] btemp = new char[this.maxLength + 1];
	    
	    short width = GEM.getFontWidth(this.font);
	    short nbC = (this.width - 5) / width;
	    
	    short ind = 0;
	    short key;
		short x = 0;
		short y = 0;
		Container ct = this.parent;

		if (!this.enabled) {
			return;
		}

		while (ct) {
			x = x + ct.x;
			y = y + ct.y;
			ct = ct.parent;
		}

		x = this.x + x + 1;
		y = this.y + y + 1;

		if (this.font != Font.FONT_SYSTEM) {
    		native {
				FontSetSys(this->font);
			}
		}

		native {
			if (strlen(this->caption->value) > this->maxLength) {
				this->setCaption_String(this, this->caption->substring_long_int_long_int(this, 0, this->maxLength));
			}
			strcpy(buffer, this->caption->value);
			ind = strlen(buffer);

			ScrRectFill(&(SCR_RECT){{x, y, x + this->width - 3, y + this->height - 3}}, ScrRect, A_REVERSE);
			this->printTextNoWarp_char_p_short_int_short_int_short_int(this, buffer, x, y, ind);
		}

	   	while (true) {
	    	key = native ( GKeyIn(NULL, 0) );

	    	if (key != KEY_ENTER) {
				if (key == KEY_BACKSPACE) {
					if (ind > 0) {
						if (ind < native ( strlen (buffer) )) {
							native {
								strcpy(&buffer[ind-1], &buffer[ind]);
							}
							ind--;
						}
						else {
							buffer[--ind] = '\0';
						}
					}
				}
				else if (key == System_KEY_DIAMOND + KEY_BACKSPACE) {
					if (ind < native ( strlen (buffer) ) ) {
						native {
							strcpy(btemp, &buffer[ind+1]);
							strcpy(&buffer[ind], btemp);
						}
					}
				}
				else if (key == System_KEY_LEFT) {
					if (ind) {
						ind--;
					}
				}
				else if (key == System_KEY_RIGHT) {
					if (ind < native ( strlen(buffer) )) {
						ind++;
					}
				}
				else if (key == System_KEY_UP) {
					ind -= nbC;
					if (ind < 0) {
						ind = 0;
					}
				}
				else if (key == System_KEY_DOWN) {
					ind += nbC;
					if (ind > native ( strlen(buffer) )) {
						ind = native ( strlen(buffer) );
					}
				}
				else if (key == System_KEY_DIAMOND + System_KEY_UP) {
					ind = 0;
				}
				else if (key == System_KEY_DIAMOND + System_KEY_DOWN) {
						ind = native ( strlen(buffer) );
				}
				else if (key == KEY_CLEAR) {
					ind = 0;
					buffer[ind] = '\0';
				}
				else {
					if (ind < this.maxLength && (key > 13 && key < 255 || key == System_KEY_DIAMOND + KEY_ENTER)) {
						native {
							strcpy(btemp, &buffer[ind]);
							strcpy(&buffer[ind+1], btemp);
						}
						if (key != System_KEY_DIAMOND + KEY_ENTER) {
							buffer[ind++] = key;
						}
						else {
							buffer[ind++] = '\n';
						}
					}
				}
				native {
					//if (ind < max) {
						ScrRectFill(&(SCR_RECT){{x, y, x + this->width - 3, y + this->height - 3}}, ScrRect, A_REVERSE);
						this->printTextNoWarp_char_p_short_int_short_int_short_int(this, buffer, x, y, ind);
						//}
				}
	    	}
	    	else {
	    		break;
	    	}
	    }

		if (this.font != Font.FONT_SYSTEM) {
			native {
				FontSetSys(GEM_gem->systemFont);
			}
		}

		this.setCaption(new String(Character.copyValueOf((char*)buffer), false));
		
		System.free(buffer);
	    System.free(btemp);
		
		super.use();
	}
}